#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
02-use-finetuned-model-for-predictions.py
Purpose: load a previously fine-tuned model and use it make predictions.
@author: Andreu Casas (acs706@nyu.edu)
"""

# MODULES
#==============================================================================
from __future__ import print_function, division
import torch
import torch.nn as nn
from torch.autograd import Variable
from torchvision import datasets, models
import matplotlib.pyplot as plt
from PIL import Image
plt.ion()   # interactive mode
import os
# - DEVELOPMENT: move to the directory where the local copy of vpgtools is
# - PRODUCTION: no need to move to that directory
os.chdir('/Users/andreucasas/Desktop/repos/')
import vpgtools.vpgtools as vpg


# PATHS & CONSTANTS
#==============================================================================
IMG_DIR = '/Users/andreucasas/Desktop/repos/vpgtools/data/protest_exp/test/positive/'
MODEL_PATH = '/Users/andreucasas/Desktop/repos/vpgmodels/protest_model.pth.tar'

# DATA
#==============================================================================
# - load fine-tuned model: in this case I'm loading a finetuned version of 
#       ResNet18.
model = models.resnet18(pretrained = True)
num_ftrs = model.fc.in_features
model.fc = nn.Linear(num_ftrs, 2)
model.load_state_dict(torch.load(MODEL_PATH))

# - loading a set of images to classify 
img_list = os.listdir(IMG_DIR)
dataset = []
for img_file in img_list:
    img = Image.open('%s%s'%(IMG_DIR, img_file)).convert('RGB')
    dataset.append(img)


# DATA WRANGLING
#==============================================================================
# - specifying the transformation-preprocessingg to apply to the images
preprocess = vpg.data_tranform(train = False)

# - checking if the machine has GPUs and enabling GPU computing if so
use_gpu = torch.cuda.is_available()

# MAIN: Peform Predictions
#==============================================================================
# - iterate through loaded iamges
for img in dataset:
    var = Variable(preprocess(img).unsqueeze(0))
    pred = model(var)
    probs, indeces = nn.Softmax()(pred).data.sort()
    print(probs)